#!/usr/bin/perl -w

use strict;
use URI::Escape;

# Extract URLs of Google search results from results page.  The web browser
# lynx is used to render the HTML.  The last part of the file name has to
# contain the number of URLs it contains.
# -> file name of HTML search results page
# <- array of URLs

sub lgetglinks
{
  my $fname= $_[0];
  my $nresults;
  my @results;
  my @links;
  my @tlinks;
  my @parts;
  my $TAGCHR= qr/[^>"]|"[^"]*"/;
  my $threshlen= 80;

  $nresults= $1 if $fname =~ /-(\d+)(?:-|\.)/;
  open SRC, "<$fname";
  my $html;
  {
    local $/;
    $html= <SRC>;
    close SRC;
  }
  # Eliminate cache links up-front: (may have to be extended to other
  # languages)
  $html =~ s/<a\b$TAGCHR+\bhref=[^>]+\bcache\b[^>]+>(?:Cached|Im&nbsp;Cache)<\/a>//ig;
  # First extract <a href=...> links:
  @parts= split /<a\b$TAGCHR+\bhref\s*=\s*/i, $html;
  shift @parts;
  for my $part (@parts) {
    $part =~ s/^"([^"]*)".*?>// || $part =~ s/^([^>]*)>// || next;
    my $link= $1;
    # Handle links redirected via google:
    if( $link =~ /^(?:http:\/\/www.google.com)?\/url\?q=(\w+:\/\/[^&]+)/ ) {
      $link= $1;
    }
    next unless $link =~ /^\w+:\/\//;		      # discard relative links
    # Discard suggested searches, translations, advertisements...:
    next if $link =~ /^\w+:\/\/[^\/]*\bgoogle\b[^\/]*\/
	       (?:search|videosearch|pagead|translate|news|article|url\?sa=|.*\badurl=)/ix;
    # Discard links to Google server by numerical IP:
    next if $link =~ /^\w+:\/\/209\.85\./;
    # Google books links which count among the results have a sig=...
    # field, discard the others
    next if $link =~ /\bbooks\.google\.com\b/ && $link !~ /\bsig=/;
# Youtube "watch" links reinstated.  Video links seem to count as a single text
# link even if there are several; counts of older data series suggest they are
# needed to tally with the requested number of links even if with some recent
# series they do not.
    # Discard youtube search results link
    next if $link =~ /^http:\/\/www.youtube.com\/results\b/;
    $link =~ s/&lt;/</g;
    $link =~ s/&gt;/>/g;
    $link =~ s/&quot;/"/g;
    $link =~ s/&amp;/&/g;
    # Discard duplicate links
    next if @links && uri_unescape($links[-1]) eq uri_unescape($link);
#    print "*** found link $link\n";
    push @links, $link;
  }
  # Second, extract text possibly giving result URL based on which tags
  # typically precede it:
  # Tags preceding text giving result URLs: (may have to be extended)
  @parts= split /<cite>|<br>|<span\s+class=(?:"a"|a)>|<font\s+[^>]+>/i, $html;
  for my $part (@parts) {
    # Tags to ignore in text giving result URL: (may have to be extended)
    $part =~ s/<b>|<\/b>|<wbr>//gi;
    $part =~ s/<$TAGCHR+>/ /go;
    $part =~ /^\s*([^\s]*\w[^\w\s]\w[^\s]*)\s/ or next;
    my $txlink= $1;
    # Discard unasked-for Google maps link:
    next if $txlink eq "maps.google.com";
#    print $txlink if $txlink =~ /books\.google\.com/;
    # Discard Google books link unless referring to a specific book:
    next if $txlink eq "books.google.com";
    $txlink =~ s/&lt;/</g;
    $txlink =~ s/&gt;/>/g;
    $txlink =~ s/&quot;/"/g;
    $txlink =~ s/&amp;/&/g;
#    print "+++ found tlink $txlink\n";
    push @tlinks, [ split(/\.\.\./, $txlink) ];
  }
  # Third, return only the links for which a corresponding text giving all or
  # part of the URL exists:
linkcheck:
  for my $link (@links) {
    my $ulink= uri_unescape($link);
trytext:
    for my $tlink (@tlinks) {
      next unless defined($tlink);
      for my $t (@$tlink) {
	# The text links may or may not have URI escapes, so try both
	unless( index($ulink, $t) >= 0 || index($ulink, uri_unescape($t)) >= 0 )
	{
	  # Google books links which are part of the regular results are
	  # sometimes linked by ID but named by ISBN
	  unless( $ulink =~ /books\.google\.com\/books\?id=/  &&
	      	  $t =~ /books\.google\.com\/books\?isbn=/ ) {
	    next trytext;
	  }
	}
      }
      $tlink= undef;
      push @results, $link;
      next linkcheck;
    }
  }
  if( defined($nresults) && scalar(@results) != $nresults ) {
    print "Warning: Found ", scalar(@results), " links in $fname\n";
  }
  return @results;
}

for my $file (@ARGV) {
#  print "=== $file ===\n";
  my @glinks= lgetglinks($file);
  print join("\n", @glinks), "\n";
}

